/***************************************************************************
 *   Copyright (C) 2005-2011 LpzRobots development team                    *
 *    Georg Martius  <georg dot martius at web dot de>                     *
 *    Frank Guettler <guettler at informatik dot uni-leipzig dot de        *
 *    Frank Hesse    <frank at nld dot ds dot mpg dot de>                  *
 *    Ralf Der       <ralfder at mis dot mpg dot de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/

#include "camerasensor.h"

namespace lpzrobots {


  CameraSensor::CameraSensor()
    : camera(0), isInitDataSet(false) {    
  }

  void CameraSensor::setInitData(Camera* camera, const OdeHandle& odeHandle, 
                                 const OsgHandle& osgHandle, const osg::Matrix& pose){
    assert(camera);
    this->camera    = camera;
    this->odeHandle = odeHandle;
    this->osgHandle = osgHandle;
    this->pose      = pose;
    isInitDataSet   = true;
  }


  CameraSensor::~CameraSensor() {
    if(camera) delete camera;
  }  

  /// changes the relative pose of the camera
  void CameraSensor::setPose(const osg::Matrix& pose){
    if(camera)
      camera->setPose(pose);
  }
  
  /// returns the relative pose of the camera
  osg::Matrix CameraSensor::getPose(){
    if(camera)
      return camera->getPose();
    else
      return osg::Matrix();
  }
  
  std::list<sensor> CameraSensor::CameraSensor::get() const {
    int num = getSensorNumber();
    sensor* s = new sensor[num];
    get(s,num);
    std::list<sensor> result(s,s+num);
    delete[] s;
    return result;
  }

  void CameraSensor::init(Primitive* own){
    assert(isInitDataSet && "initialize the camerasensor with setInitData()!");
    camera->init(odeHandle, osgHandle, own, pose);
    intern_init();
  }

  /// we update the camera's visual appearance
  void CameraSensor::update(){
    camera->update();
  }



}
