/***************************************************************************
 *   Copyright (C) 2005-2011 LpzRobots development team                    *
 *    Georg Martius  <georg dot martius at web dot de>                     *
 *    Frank Guettler <guettler at informatik dot uni-leipzig dot de        *
 *    Frank Hesse    <frank at nld dot ds dot mpg dot de>                  *
 *    Ralf Der       <ralfder at mis dot mpg dot de>                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 ***************************************************************************/
#ifndef __SELECTIVENOISEWIRING_H
#define __SELECTIVENOISEWIRING_H

#include <selforg/one2onewiring.h>
#include <vector>

/** 
 * Implements a one to one wiring 
 *  and allows to select the noise strength per sensor channel
 */
class SelectiveNoiseWiring : public One2OneWiring {
public:
  /** constructor
      @param noise NoiseGenerator that is used for adding noise to motor values  
      @param noiseStrengths for each channel the noise strength 
       (no noise if channel is out of array bounds)
      @param plotMode see AbstractWiring 
  */
  SelectiveNoiseWiring(NoiseGenerator* noise, std::vector<double> noiseStrengths, 
                       int plotMode=Controller)
    : One2OneWiring(noise, plotMode), noiseStrengths(noiseStrengths) {    
  }
  virtual ~SelectiveNoiseWiring(){}
  
  virtual bool wireSensorsIntern(const sensor* rsensors, int rsensornumber, 
                                 sensor* csensors, int csensornumber, double noise){
    assert(rsensornumber == this->rsensornumber);
    assert(csensornumber == this->csensornumber);
    // the noisevals are set in abstractwiring
    for(int i=0; i< rsensornumber; i++){
      csensors[i] = rsensors[i] + getNoiseStrength(i)*noisevals[i];
    }
    return true;  
  }
  
protected:
  double getNoiseStrength(unsigned int i){
    if(i>=noiseStrengths.size()) return 0;
    else return noiseStrengths[i];
  }

  std::vector<double> noiseStrengths;

};

#endif
